#!/usr/bin/env python
# -*- coding: utf-8 -*-

# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release: dinasset_gegl.py by paynekj http://gimpchat.com/viewtopic.php?f=9&t=18040&p=254914#p254914

from gimpfu import *
from sys import argv, platform
from ctypes import *
import random
import math
import sys, os

def load_library (library_name):
    if platform == "linux" or platform == "linux2":
        library_name = library_name + '.so.0'
    elif platform == "win32":
        from ctypes.util import find_library
        library_name = find_library (library_name + "-0")
    else:
        raise BaseException ("TODO")
    return CDLL (library_name)

gimp = load_library ('libgimp-2.0')

def gegl_graph(image, drawable, gegl_graph_string):
    class GeglBuffer(Structure):
        pass
    drawable_id = drawable.ID
    
    gegl = load_library ('libgegl-0.4')
    sucess = gegl.gegl_init (None, None)
    gimp.gimp_drawable_get_shadow_buffer.restype = POINTER (GeglBuffer)
    gimp.gimp_drawable_get_buffer.restype        = POINTER (GeglBuffer)

    x = c_int (); y = c_int (); w = c_int (); h = c_int ()
    non_empty,x,y,w,h = pdb.gimp_drawable_mask_intersect (drawable)
    args = [b"string", c_char_p (gegl_graph_string), c_void_p ()]
    
    if non_empty:
        source = gimp.gimp_drawable_get_buffer (drawable_id)
        target = gimp.gimp_drawable_get_shadow_buffer (drawable_id)
        sucess = gegl.gegl_render_op (source, target, "gegl:gegl", *args)
        gegl.gegl_buffer_flush (target)
        gimp.gimp_drawable_merge_shadow (drawable_id, PushUndo = True)
        gimp.gimp_drawable_update (drawable_id, x, y, w, h)
        gimp.gimp_displays_flush ()

register(
   "python_fu_gegl_graph",
   "Do GEGL graph example Tooltip",
   "Do GEGL graph example Additional",
   "paynekj",
   "GPL3",
   "2020.08.24",
   "Do GEGL graph", 
   "RGB*",
   [
     (PF_IMAGE,      "image",       "Input image", None),
     (PF_DRAWABLE,   "drawable", "Input drawable", None),
     (PF_STRING, "gegl_graph_string", "GEGL Graph String", "gegl:wind")
   ],
   [],
   gegl_graph,
   menu="<Image>/GEGL as Python/Gegl Graph/"
   )

#---------------------------------------------------------------------------------------------
#| Change Log |
# ------------
# Rel 1: Tin Tran 72 out of 256 code sections generated http://gimpchat.com/viewtopic.php?f=9&t=20613&start=10#p284958
# Rel 2: MrQ 
# - combined into one file gegl-python-fu.py and gegl-graph.py to gegl-python-fu.py & gegl-graph.py (standalone files: gegl-python-fu.py and gegl-graph.py must be deleted)
# - change menu location (you can change it very quickly as you want using Notepad++),
# - removed gegl_tile (because it doesn't work), gegl-nop (No operation), 
# - added: bump-map, component_extract, dropshadow, embos, supernova,mosaic
# Rel 2.1: MrQ only bugs-fix
# Rel 3.0: MrQ - added: color-exchange, cubism, long-shadow, newsprint, pixelize.

#-------------------------------------MrQ-----------------------------------------------------
def rgb_to_hex(rgb_color):
    r = rgb_color[0]
    g = rgb_color[1]
    b = rgb_color[2]
    return '#%02x%02x%02x' % (r,g,b)
	
#---------------------------------------------------------------------------------------------

def gegl_absolute(image, layer, ):

    gegl_graph_string="gegl:absolute " % ()
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_absolute",
    "GEGL to Python. Works on the active layer",
    "Makes each linear RGB component be the absolute of its value",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-absolute...",             #Menu path
    "*", 
    [

    ],
    [],
    gegl_absolute)
#---------------------------------------------------------------------------------------------	

def gegl_antialias(image, layer, ):

    gegl_graph_string="gegl:antialias " % ()
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_antialias",
    "GEGL to Python. Works on the active layer",
    "Antialias using the Scale3X edge-extrapolation algorithm",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-antialias...",             #Menu path
    "*", 
    [

    ],
    [],
    gegl_antialias)

#---------------------------------------------------------------------------------------------
def gegl_bilateral_filter(image, layer, blur_radius,edge_preservation):

    gegl_graph_string="gegl:bilateral-filter blur-radius=%f edge-preservation=%f " % (blur_radius,edge_preservation)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_bilateral_filter",
    "GEGL to Python. Works on the active layer",
    "Like a gaussian blur; but where the contribution for each neighborhood pixel is also weighted by the color difference with the original center pixel.",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-bilateral-filter...",             #Menu path
    "*", 
    [
    (PF_FLOAT, "blur_radius","blur-radius (default:4.0, 0.0..100.0):", 4.0),
    (PF_FLOAT, "edge_preservation","edge-preservation (default:8.0, 0.0..100.0):", 8.0)
    ],
    [],
    gegl_bilateral_filter)

#---------------------------------------------------------------------------------------------
def gegl_bloom(image, layer, threshold,softness,radius,strength,limit_exposure):

    gegl_graph_string="gegl:bloom threshold=%f softness=%f radius=%f strength=%f limit-exposure=%d " % (threshold,softness,radius,strength,limit_exposure)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_bloom",
    "GEGL to Python. Works on the active layer",
    "Add glow around bright areas",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-bloom...",             #Menu path
    "*", 
    [
    (PF_FLOAT, "threshold","threshold (default:50.0, 0.0..100.0):", 50.0),
    (PF_FLOAT, "softness","softness (default:25.0, 0.0..100.0):", 25.0),
    (PF_FLOAT, "radius","radius (default:10.0, 0.0..100.0):", 10.0),
    (PF_FLOAT, "strength","strength (default:50.0, 0.0..100.0):", 50.0),
    (PF_TOGGLE, "limit_exposure","limit-exposure (default:True):",True)
    ],
    [],
    gegl_bloom)

#---------------------------------------------------------------------------------------------
def gegl_box_blur(image, layer, radius):

    gegl_graph_string="gegl:box-blur radius=%d " % (radius)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_box_blur",
    "GEGL to Python. Works on the active layer",
    "Blur resulting from averaging the colors of a square neighborhood.",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-box-blur...",             #Menu path
    "*", 
    [
    (PF_SPINNER, "radius", "radius (default:4, 0..100):", 4, (0, 100, 1))
    ],
    [],
    gegl_box_blur)

#---------------------------------------------------------------------------------------------
def gegl_brightness_contrast(image, layer, brightness):

    gegl_graph_string="gegl:brightness-contrast brightness=%f " % (brightness)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_brightness_contrast",
    "GEGL to Python. Works on the active layer",
    "Changes the light level and contrast. This operation operates in linear light, 'contrast' is a scale factor around 50% gray, and 'brightness' a constant offset to apply after contrast scaling.",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-brightness-contrast...",             #Menu path
    "*", 
    [
    (PF_FLOAT, "brightness","brightness (default:0.0, -1.0..1.0):", 0.0)
    ],
    [],
    gegl_brightness_contrast)

#---------------------------------------------------------------------------------------------
def gegl_c2g(image, layer, radius,samples,iterations,enhance_shadows):

    gegl_graph_string="gegl:c2g radius=%d samples=%d iterations=%d enhance-shadows=%d " % (radius,samples,iterations,enhance_shadows)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_c2g",
    "GEGL to Python. Works on the active layer",
    "Color to grayscale conversion, uses envelopes formed with the STRESS approach to perform local color-difference preserving grayscale generation.",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-c2g (Color to Gray )...",             #Menu path
    "*", 
    [
    (PF_SPINNER, "radius", "radius (default:300, 2..6000):", 300, (2, 6000, 1)),
    (PF_SPINNER, "samples", "samples (default:4, 1..1000):", 4, (1, 1000, 1)),
    (PF_SPINNER, "iterations", "iterations (default:10, 1..1000):", 10, (1, 1000, 1)),
    (PF_TOGGLE,  "enhance_shadows","enhance-shadows (default:True):",True)
    ],
    [],
    gegl_c2g)

#---------------------------------------------------------------------------------------------
def gegl_cartoon(image, layer, mask_radius,pct_black):

    gegl_graph_string="gegl:cartoon mask-radius=%f pct-black=%f " % (mask_radius,pct_black)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_cartoon",
    "GEGL to Python. Works on the active layer",
    "Simulates a cartoon, its result is similar to a black felt pen drawing subsequently shaded with color. This is achieved by enhancing edges and darkening areas that are already distinctly darker than their neighborhood",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-cartoon...",             #Menu path
    "*", 
    [
    (PF_FLOAT, "mask_radius","mask-radius (default:7.0, 0.0..50.0):", 7.0),
    (PF_FLOAT, "pct_black","pct-black (default:0.2, 0.0..1.0):", 0.2)
    ],
    [],
    gegl_cartoon)

#---------------------------------------------------------------------------------------------
def gegl_channel_mixer(image, layer, preserve_luminosity,rr_gain,rg_gain,rb_gain,gr_gain,gg_gain,gb_gain,br_gain,bg_gain,bb_gain):

    gegl_graph_string="gegl:channel-mixer preserve-luminosity=%d rr-gain=%f rg-gain=%f rb-gain=%f gr-gain=%f gg-gain=%f gb-gain=%f br-gain=%f bg-gain=%f bb-gain=%f " % (preserve_luminosity,rr_gain,rg_gain,rb_gain,gr_gain,gg_gain,gb_gain,br_gain,bg_gain,bb_gain)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_channel_mixer",
    "GEGL to Python. Works on the active layer",
    "Remix colors; by defining relative contributions from source components.",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-channel-mixer...",             #Menu path
    "*", 
    [
    (PF_TOGGLE, "preserve_luminosity","preserve-luminosity (default:True):",True),
    (PF_FLOAT, "rr_gain","rr-gain (default:1.0, -2.0..2.0):", 1.0),
    (PF_FLOAT, "rg_gain","rg-gain (default:0.0, -2.0..2.0):", 0.0),
    (PF_FLOAT, "rb_gain","rb-gain (default:0.0, -2.0..2.0):", 0.0),
    (PF_FLOAT, "gr_gain","gr-gain (default:0.0, -2.0..2.0):", 0.0),
    (PF_FLOAT, "gg_gain","gg-gain (default:1.0, -2.0..2.0):", 1.0),
    (PF_FLOAT, "gb_gain","gb-gain (default:0.0, -2.0..2.0):", 0.0),
    (PF_FLOAT, "br_gain","br-gain (default:0.0, -2.0..2.0):", 0.0),
    (PF_FLOAT, "bg_gain","bg-gain (default:0.0, -2.0..2.0):", 0.0),
    (PF_FLOAT, "bb_gain","bb-gain (default:1.0, -2.0..2.0):", 1.0)
    ],
    [],
    gegl_channel_mixer)

#---------------------------------------------------------------------------------------------
def gegl_clone(image, layer, ref):

    gegl_graph_string="gegl:clone ref=%s " % (ref)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_clone",
    "GEGL to Python. Works on the active layer",
    "Clone a buffer, this is the same as gegl:nop but can get special treatment to get more human readable references in serializations/UI.",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-clone...",             #Menu path
    "*", 
    [
    (PF_STRING, "ref", "ref(default:ID):", "ID")
    ],
    [],
    gegl_clone)

#---------------------------------------------------------------------------------------------
def gegl_color_assimilation_grid(image, layer, grid_size,saturation,angle,line_thickness):

    gegl_graph_string="gegl:color-assimilation-grid grid-size=%f saturation=%f angle=%f line-thickness=%f " % (grid_size,saturation,angle,line_thickness)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_color_assimilation_grid",
    "GEGL to Python. Works on the active layer",
    "Turn image grayscale and overlay an oversaturated grid - through color assimilation happening in the human visual system, for some grid scales this produces the illusion that the grayscale grid cells themselves also have color",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-color-assimilation-grid...",             #Menu path
    "*", 
    [
    (PF_FLOAT, "grid_size","grid-size (default:23.0, 0.0..40.0):", 23.0),
    (PF_FLOAT, "saturation","saturation (default:2.5, 0.0..10.0):", 2.5),
    (PF_FLOAT, "angle","angle (default:45.0, -180.0..180.0):", 45.0),
    (PF_FLOAT, "line_thickness","line-thickness (default:0.4, 0.0..1.0):", 0.4)
    ],
    [],
    gegl_color_assimilation_grid)

#---------------------------------------------------------------------------------------------
def gegl_color_enhance(image, layer, ):

    gegl_graph_string="gegl:color-enhance " % ()
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_color_enhance",
    "GEGL to Python. Works on the active layer",
    "Stretch color chroma to cover maximum possible range, keeping hue and lightness untouched.",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-color-enhance...",             #Menu path
    "*", 
    [

    ],
    [],
    gegl_color_enhance)

#---------------------------------------------------------------------------------------------
def gegl_color_temperature(image, layer, original_temperature,intended_temperature):

    gegl_graph_string="gegl:color-temperature original-temperature=%f intended-temperature=%f " % (original_temperature,intended_temperature)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_color_temperature",
    "GEGL to Python. Works on the active layer",
    "Change the color temperature of the image, from an assumed original color temperature to an intended one.",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-color-temperature...",             #Menu path
    "*", 
    [
    (PF_FLOAT, "original_temperature","original-temperature (default:6500.0, 1000.0..12000.0):", 6500.0),
    (PF_FLOAT, "intended_temperature","intended-temperature (default:6500.0, 1000.0..12000.0):", 6500.0)
    ],
    [],
    gegl_color_temperature)

#---------------------------------------------------------------------------------------------
def gegl_difference_of_gaussians(image, layer, radius1,radius2):

    gegl_graph_string="gegl:difference-of-gaussians radius1=%f radius2=%f " % (radius1,radius2)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_difference_of_gaussians",
    "GEGL to Python. Works on the active layer",
    "Edge detection with control of edge thickness, based on the difference of two gaussian blurs",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-difference-of-gaussians...",             #Menu path
    "*", 
    [
    (PF_FLOAT, "radius1","radius1 (default:1.0, 0.0..10.0):", 1.0),
    (PF_FLOAT, "radius2","radius2 (default:2.0, 0.0..20.0):", 2.0)
    ],
    [],
    gegl_difference_of_gaussians)

#---------------------------------------------------------------------------------------------
def gegl_diffraction_patterns(image, layer, red_frequency,green_frequency,blue_frequency,red_contours,green_contours,blue_contours,red_sedges,green_sedges,blue_sedges,brightness,scattering,polarization,width,height):

    gegl_graph_string="gegl:diffraction-patterns red-frequency=%f green-frequency=%f blue-frequency=%f red-contours=%f green-contours=%f blue-contours=%f red-sedges=%f green-sedges=%f blue-sedges=%f brightness=%f scattering=%f polarization=%f width=%d height=%d " % (red_frequency,green_frequency,blue_frequency,red_contours,green_contours,blue_contours,red_sedges,green_sedges,blue_sedges,brightness,scattering,polarization,width,height)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_diffraction_patterns",
    "GEGL to Python. Works on the active layer",
    "Generate diffraction patterns",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-diffraction-patterns...",             #Menu path
    "*", 
    [
    (PF_FLOAT, "red_frequency","red-frequency (default:0.81, 0.0..20.0):", 0.81),
    (PF_FLOAT, "green_frequency","green-frequency (default:1.22, 0.0..20.0):", 1.22),
    (PF_FLOAT, "blue_frequency","blue-frequency (default:1.12, 0.0..20.0):", 1.12),
    (PF_FLOAT, "red_contours","red-contours (default:0.82, 0.0..10.0):", 0.82),
    (PF_FLOAT, "green_contours","green-contours (default:0.82, 0.0..10.0):", 0.82),
    (PF_FLOAT, "blue_contours","blue-contours (default:0.97, 0.0..10.0):", 0.97),
    (PF_FLOAT, "red_sedges","red-sedges (default:0.61, 0.0..1.0):", 0.61),
    (PF_FLOAT, "green_sedges","green-sedges (default:0.68, 0.0..1.0):", 0.68),
    (PF_FLOAT, "blue_sedges","blue-sedges (default:0.64, 0.0..1.0):", 0.64),
    (PF_FLOAT, "brightness","brightness (default:0.07, 0.0..1.0):", 0.07),
    (PF_FLOAT, "scattering","scattering (default:37.13, 0.0..100.0):", 37.13),
    (PF_FLOAT, "polarization","polarization (default:-0.47, -1.0..1.0):", -0.47),
    (PF_SPINNER, "width", "width (default:200, 0..4096):", 200, (0, 4096, 1)),
    (PF_SPINNER, "height", "height (default:200, 0..4096):", 200, (0, 4096, 1))
    ],
    [],
    gegl_diffraction_patterns)

#---------------------------------------------------------------------------------------------
def gegl_display(image, layer, window_title):

    gegl_graph_string="gegl:display window-title=%s " % (window_title)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_display",
    "GEGL to Python. Works on the active layer",
    "Display the input buffer in a window.",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-display...",             #Menu path
    "*", 
    [
    (PF_STRING, "window_title", "window-title(default:window_title):", "window_title")
    ],
    [],
    gegl_display)

#---------------------------------------------------------------------------------------------
def gegl_domain_transform(image, layer, n_iterations,spatial_factor,edge_preservation):

    gegl_graph_string="gegl:domain-transform n-iterations=%d spatial-factor=%f edge-preservation=%f " % (n_iterations,spatial_factor,edge_preservation)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_domain_transform",
    "GEGL to Python. Works on the active layer",
    "An edge-preserving smoothing filter implemented with the Domain Transform recursive technique. Similar to a bilateral filter, but faster to compute",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-domain-transform...",             #Menu path
    "*", 
    [
    (PF_SPINNER, "n_iterations", "n-iterations (default:3, 1..5):", 3, (1, 5, 1)),
    (PF_FLOAT, "spatial_factor","spatial-factor (default:30.0, 0.0..1000.0):", 30.0),
    (PF_FLOAT, "edge_preservation","edge-preservation (default:0.8, 0.0..1.0):", 0.8)
    ],
    [],
    gegl_domain_transform)

#---------------------------------------------------------------------------------------------
def gegl_edge_laplace(image, layer, ):

    gegl_graph_string="gegl:edge-laplace " % ()
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_edge_laplace",
    "GEGL to Python. Works on the active layer",
    "High-resolution edge detection",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-edge-laplace...",             #Menu path
    "*", 
    [

    ],
    [],
    gegl_edge_laplace)

#---------------------------------------------------------------------------------------------
def gegl_edge_neon(image, layer, radius,amount):

    gegl_graph_string="gegl:edge-neon radius=%f amount=%f " % (radius,amount)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_edge_neon",
    "GEGL to Python. Works on the active layer",
    "Performs edge detection using a Gaussian derivative method",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-edge-neon...",             #Menu path
    "*", 
    [
    (PF_FLOAT, "radius","radius (default:5.0, 0.0..50.0):", 5.0),
    (PF_FLOAT, "amount","amount (default:0.0, 0.0..3.0):", 0.0)
    ],
    [],
    gegl_edge_neon)

#---------------------------------------------------------------------------------------------
def gegl_edge_sobel(image, layer, horizontal,vertical,keep_sign):

    gegl_graph_string="gegl:edge-sobel horizontal=%d vertical=%d keep-sign=%d " % (horizontal,vertical,keep_sign)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_edge_sobel",
    "GEGL to Python. Works on the active layer",
    "Specialized direction-dependent edge detection",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-edge-sobel...",             #Menu path
    "*", 
    [
    (PF_TOGGLE, "horizontal","horizontal (default:True):",True),
    (PF_TOGGLE, "vertical","vertical (default:True):",True),
    (PF_TOGGLE, "keep_sign","keep-sign (default:True):",True)
    ],
    [],
    gegl_edge_sobel)

#---------------------------------------------------------------------------------------------
def gegl_engrave(image, layer, row_height,limit):

    gegl_graph_string="gegl:engrave row-height=%d limit=%d " % (row_height,limit)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_engrave",
    "GEGL to Python. Works on the active layer",
    "Simulate an antique engraving",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-engrave...",             #Menu path
    "*", 
    [
    (PF_SPINNER, "row_height", "row-height (default:10, 2..16):", 10, (2, 16, 1)),
    (PF_TOGGLE, "limit","limit (default:True):",True)
    ],
    [],
    gegl_engrave)

#---------------------------------------------------------------------------------------------
def gegl_exposure(image, layer, black_level,exposure):

    gegl_graph_string="gegl:exposure black-level=%f exposure=%f " % (black_level,exposure)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_exposure",
    "GEGL to Python. Works on the active layer",
    "Change exposure of an image in shutter speed stops",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-exposure...",             #Menu path
    "*", 
    [
    (PF_FLOAT, "black_level","black-level (default:0.0, -0.1..0.1):", 0.0),
    (PF_FLOAT, "exposure","exposure (default:0.0, -10.0..10.0):", 0.0)
    ],
    [],
    gegl_exposure)

#---------------------------------------------------------------------------------------------
def gegl_fattal02(image, layer, alpha,beta,saturation,noise):

    gegl_graph_string="gegl:fattal02 alpha=%f beta=%f saturation=%f noise=%f " % (alpha,beta,saturation,noise)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_fattal02",
    "GEGL to Python. Works on the active layer",
    "Adapt an image, which may have a high dynamic range, for presentation using a low dynamic range. This operator attenuates the magnitudes of local image gradients, producing luminance within the range 0.0-1.0.\n This tonemapping approach was originally presented by Raanan Fattal, in the 2002 SIGGRAPH paper: Gradient Domain High Dynamic Range Compression.",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-fattal02...",             #Menu path
    "*", 
    [
    (PF_FLOAT, "alpha","alpha (default:1.0, 0.0..2.0):", 1.0),
    (PF_FLOAT, "beta","beta (default:0.9, 0.1..2.0):", 0.9),
    (PF_FLOAT, "saturation","saturation (default:0.8, 0.0..1.0):", 0.8),
    (PF_FLOAT, "noise","noise (default:0.0, 0.0..1.0):", 0.0)
    ],
    [],
    gegl_fattal02)

#---------------------------------------------------------------------------------------------
def gegl_gegl(image, layer, string,error):

    gegl_graph_string="gegl:gegl string=%s error=%s " % (string,error)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_gegl",
    "GEGL to Python. Works on the active layer",
    "Do a chain of operations, with key=value pairs after each operation name to set properties. And aux=[ source filter ] for specifying a chain with a source as something connected to an aux pad.",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-gegl...",             #Menu path
    "*", 
    [
    (PF_STRING, "string", "string(default:#):", "#"),
    (PF_STRING, "error", "error(default:error:true):", "error:true")
    ],
    [],
    gegl_gegl)

#---------------------------------------------------------------------------------------------
def gegl_gray(image, layer, ):

    gegl_graph_string="gegl:gray " % ()
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_gray",
    "GEGL to Python. Works on the active layer",
    "Turns the image grayscale",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-gray...",             #Menu path
    "*", 
    [

    ],
    [],
    gegl_gray)

#---------------------------------------------------------------------------------------------
def gegl_high_pass(image, layer, std_dev,contrast):

    gegl_graph_string="gegl:high-pass std-dev=%f contrast=%f " % (std_dev,contrast)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_high_pass",
    "GEGL to Python. Works on the active layer",
    "Enhances fine details.",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-high-pass...",             #Menu path
    "*", 
    [
    (PF_FLOAT, "std_dev","std-dev (default:4.0, 0.0..1000.0):", 4.0),
    (PF_FLOAT, "contrast","contrast (default:1.0, 0.0..5.0):", 1.0)
    ],
    [],
    gegl_high_pass)

#---------------------------------------------------------------------------------------------
def gegl_hue_chroma(image, layer, hue,chroma,lightness):

    gegl_graph_string="gegl:hue-chroma hue=%f chroma=%f lightness=%f " % (hue,chroma,lightness)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_hue_chroma",
    "GEGL to Python. Works on the active layer",
    "Adjust LCH Hue, Chroma, and Lightness",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-hue-chroma...",             #Menu path
    "*", 
    [
    (PF_FLOAT, "hue","hue (default:0.0, -180.0..180.0):", 0.0),
    (PF_FLOAT, "chroma","chroma (default:0.0, -100.0..100.0):", 0.0),
    (PF_FLOAT, "lightness","lightness (default:0.0, -100.0..100.0):", 0.0)
    ],
    [],
    gegl_hue_chroma)

#---------------------------------------------------------------------------------------------
def gegl_invert_gamma(image, layer, ):

    gegl_graph_string="gegl:invert-gamma " % ()
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_invert_gamma",
    "GEGL to Python. Works on the active layer",
    "Invert the components (except alpha) perceptually, the result is the corresponding 'negative' image.",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-invert-gamma...",             #Menu path
    "*", 
    [

    ],
    [],
    gegl_invert_gamma)

#---------------------------------------------------------------------------------------------
def gegl_invert_linear(image, layer, ):

    gegl_graph_string="gegl:invert-linear " % ()
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_invert_linear",
    "GEGL to Python. Works on the active layer",
    "Invert the components (except alpha) in linear light, the result is the corresponding 'negative' image.",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-invert-linear...",             #Menu path
    "*", 
    [

    ],
    [],
    gegl_invert_linear)

#---------------------------------------------------------------------------------------------
def gegl_json(image, layer, height,width):

    gegl_graph_string="gegl:json height=%f width=%f " % (height,width)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_json",
    "GEGL to Python. Works on the active layer",
    "GEGL to Python. Works on the active layer",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-json...",             #Menu path
    "*", 
    [
    (PF_FLOAT, "height","height (default:0.0, -10000..10000):", 0.0),
    (PF_FLOAT, "width","width (default:0.0, -10000..10000):", 0.0)
    ],
    [],
    gegl_json)

#---------------------------------------------------------------------------------------------
def gegl_lens_flare(image, layer, pos_x,pos_y):

    gegl_graph_string="gegl:lens-flare pos-x=%f pos-y=%f " % (pos_x,pos_y)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_lens_flare",
    "GEGL to Python. Works on the active layer",
    "Adds a lens flare effect.",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-lens-flare...",             #Menu path
    "*", 
    [
    (PF_FLOAT, "pos_x","pos-x (default:0.5, 0.0..1.0):", 0.5),
    (PF_FLOAT, "pos_y","pos-y (default:0.5, 0.0..1.0):", 0.5)
    ],
    [],
    gegl_lens_flare)

#---------------------------------------------------------------------------------------------
def gegl_levels(image, layer, in_low,in_high,out_low,out_high):

    gegl_graph_string="gegl:levels in-low=%f in-high=%f out-low=%f out-high=%f " % (in_low,in_high,out_low,out_high)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_levels",
    "GEGL to Python. Works on the active layer",
    "Remaps the intensity range of the image",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-levels...",             #Menu path
    "*", 
    [
    (PF_FLOAT, "in_low","in-low (default:0.0, 0.0..1.0):", 0.0),
    (PF_FLOAT, "in_high","in-high (default:1.0, 0.0..1.0):", 1.0),
    (PF_FLOAT, "out_low","out-low (default:0.0, 0.0..1.0):", 0.0),
    (PF_FLOAT, "out_high","out-high (default:1.0, 0.0..1.0):", 1.0)
    ],
    [],
    gegl_levels)

#---------------------------------------------------------------------------------------------
def gegl_linear_sinusoid(image, layer, x_period,y_period,x_amplitude,y_amplitude,x_phase,y_phase,angle,offset,exponent,x_offset,y_offset,rotation,supersampling):

    gegl_graph_string="gegl:linear-sinusoid x-period=%f y-period=%f x-amplitude=%f y-amplitude=%f x-phase=%f y-phase=%f angle=%f offset=%f exponent=%f x-offset=%f y-offset=%f rotation=%f supersampling=%d " % (x_period,y_period,x_amplitude,y_amplitude,x_phase,y_phase,angle,offset,exponent,x_offset,y_offset,rotation,supersampling)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_linear_sinusoid",
    "GEGL to Python. Works on the active layer",
    "Generate a linear sinusoid pattern",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-linear-sinusoid...",             #Menu path
    "*", 
    [
    (PF_FLOAT, "x_period","x-period (default:128.0, 0.0..256.0):", 128.0),
    (PF_FLOAT, "y_period","y-period (default:128.0, 0.0..256.0):", 128.0),
    (PF_FLOAT, "x_amplitude","x-amplitude (default:0.0, -2.0..2.0):", 0.0),
    (PF_FLOAT, "y_amplitude","y-amplitude (default:0.0, -2.0..2.0):", 0.0),
    (PF_FLOAT, "x_phase","x-phase (default:0.0, -512.0..512.0):", 0.0),
    (PF_FLOAT, "y_phase","y-phase (default:0.0, -512.0..512.0):", 0.0),
    (PF_FLOAT, "angle","angle (default:90.0, 0.0..360.0):", 90.0),
    (PF_FLOAT, "offset","offset (default:0.0, -1.0..1.0):", 0.0),
    (PF_FLOAT, "exponent","exponent (default:0.0, -2.0..2.0):", 0.0),
    (PF_FLOAT, "x_offset","x-offset (default:0.0, -512.0..512.0):", 0.0),
    (PF_FLOAT, "y_offset","y-offset (default:0.0, -512.0..512.0):", 0.0),
    (PF_FLOAT, "rotation","rotation (default:0.0, 0.0..360.0):", 0.0),
    (PF_SPINNER, "supersampling", "supersampling (default:1, 1..8):", 1, (1, 8, 1))
    ],
    [],
    gegl_linear_sinusoid)

#---------------------------------------------------------------------------------------------
def gegl_mantiuk06(image, layer, contrast,saturation,detail):

    gegl_graph_string="gegl:mantiuk06 contrast=%f saturation=%f detail=%f " % (contrast,saturation,detail)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_mantiuk06",
    "GEGL to Python. Works on the active layer",
    "Adapt an image, which may have a high dynamic range, for presentation using a low dynamic range. This operator constrains contrasts across multiple spatial frequencies, producing luminance within the range 0.0-1.0",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-mantiuk06...",             #Menu path
    "*", 
    [
    (PF_FLOAT, "contrast","contrast (default:0.1, 0.0..1.0):", 0.1),
    (PF_FLOAT, "saturation","saturation (default:0.8, 0.0..2.0):", 0.8),
    (PF_FLOAT, "detail","detail (default:1.0, 1.0..99.0):", 1.0)
    ],
    [],
    gegl_mantiuk06)

#---------------------------------------------------------------------------------------------
def gegl_mblur(image, layer, dampness):

    gegl_graph_string="gegl:mblur dampness=%f " % (dampness)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_mblur",
    "GEGL to Python. Works on the active layer",
    "Accumulating motion blur using a kalman filter, for use with video sequences of frames.",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-mblur...",             #Menu path
    "*", 
    [
    (PF_FLOAT, "dampness","dampness (default:0.95, 0.0..1.0):", 0.95)
    ],
    [],
    gegl_mblur)

#---------------------------------------------------------------------------------------------
def gegl_mean_curvature_blur(image, layer, iterations):

    gegl_graph_string="gegl:mean-curvature-blur iterations=%d " % (iterations)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_mean_curvature_blur",
    "GEGL to Python. Works on the active layer",
    "Regularize geometry at a speed proportional to the local mean curvature value",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-mean-curvature-blur...",             #Menu path
    "*", 
    [
    (PF_SPINNER, "iterations", "iterations (default:20, 0..60):", 20, (0, 60, 1))
    ],
    [],
    gegl_mean_curvature_blur)

#---------------------------------------------------------------------------------------------
def gegl_mirrors(image, layer, m_angle,r_angle,n_segs,c_x,c_y,o_x,o_y,trim_x,trim_y,input_scale,output_scale,clip,warp):

    gegl_graph_string="gegl:mirrors m-angle=%f r-angle=%f n-segs=%d c-x=%f c-y=%f o-x=%f o-y=%f trim-x=%f trim-y=%f input-scale=%f output-scale=%f clip=%d warp=%d " % (m_angle,r_angle,n_segs,c_x,c_y,o_x,o_y,trim_x,trim_y,input_scale,output_scale,clip,warp)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_mirrors",
    "GEGL to Python. Works on the active layer",
    "Create a kaleidoscope like effect.",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-mirrors (kaleidoscop)...",             #Menu path
    "*", 
    [
    (PF_FLOAT, "m_angle","m-angle (default:0.0, 0.0..180.0):", 0.0),
    (PF_FLOAT, "r_angle","r-angle (default:0.0, 0.0..360.0):", 0.0),
    (PF_SPINNER, "n_segs", "n-segs (default:6, 2..24):", 6, (2, 24, 1)),
    (PF_FLOAT, "c_x","c-x (default:0.5, 0.0..1.0):", 0.5),
    (PF_FLOAT, "c_y","c-y (default:0.5, 0.0..1.0):", 0.5),
    (PF_FLOAT, "o_x","o-x (default:0.0, -1.0..1.0):", 0.0),
    (PF_FLOAT, "o_y","o-y (default:0.0, -1.0..1.0):", 0.0),
    (PF_FLOAT, "trim_x","trim-x (default:0.0, 0.0..0.5):", 0.0),
    (PF_FLOAT, "trim_y","trim-y (default:0.0, 0.0..0.5):", 0.0),
    (PF_FLOAT, "input_scale","input-scale (default:100.0, 0.1..100.0):", 100.0),
    (PF_FLOAT, "output_scale","output-scale (default:1.0, 0.0..100.0):", 1.0),
    (PF_TOGGLE, "clip","clip (default:True):",True),
    (PF_TOGGLE, "warp","warp (default:True):",True)
    ],
    [],
    gegl_mirrors)

#---------------------------------------------------------------------------------------------
def gegl_mono_mixer(image, layer, preserve_luminosity,red,green,blue):

    gegl_graph_string="gegl:mono-mixer preserve-luminosity=%d red=%f green=%f blue=%f " % (preserve_luminosity,red,green,blue)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_mono_mixer",
    "GEGL to Python. Works on the active layer",
    "Monochrome channel mixer",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-mono-mixer...",             #Menu path
    "*", 
    [
    (PF_TOGGLE, "preserve_luminosity","preserve-luminosity (default:True):",True),
    (PF_FLOAT, "red","red (default:0.33, -2.0..2.0):", 0.33),
    (PF_FLOAT, "green","green (default:0.33, -2.0..2.0):", 0.33),
    (PF_FLOAT, "blue","blue (default:0.33, -2.0..2.0):", 0.33)
    ],
    [],
    gegl_mono_mixer)

#---------------------------------------------------------------------------------------------
def gegl_motion_blur_circular(image, layer, center_x,center_y,angle):

    gegl_graph_string="gegl:motion-blur-circular center-x=%f center-y=%f angle=%f " % (center_x,center_y,angle)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_motion_blur_circular",
    "GEGL to Python. Works on the active layer",
    "Circular motion blur",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-motion-blur-circular...",             #Menu path
    "*", 
    [
    (PF_FLOAT, "center_x","center-x (default:0.5, 0.0..1.0):", 0.5),
    (PF_FLOAT, "center_y","center-y (default:0.5, 0.0..1.0):", 0.5),
    (PF_FLOAT, "angle","angle (default:5.0, 0.0..360.0):", 5.0)
    ],
    [],
    gegl_motion_blur_circular)

#---------------------------------------------------------------------------------------------
def gegl_motion_blur_linear(image, layer, length,angle):

    gegl_graph_string="gegl:motion-blur-linear length=%f angle=%f " % (length,angle)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_motion_blur_linear",
    "GEGL to Python. Works on the active layer",
    "Blur pixels in a direction, simulates blurring caused by moving camera in a straight line during exposure.",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-motion-blur-linear...",             #Menu path
    "*", 
    [
    (PF_FLOAT, "length","length (default:10.0, 0.0..300.0):", 10.0),
    (PF_FLOAT, "angle","angle (default:0.0, -180.0..180.0):", 0.0)
    ],
    [],
    gegl_motion_blur_linear)

#---------------------------------------------------------------------------------------------
def gegl_motion_blur_zoom(image, layer, center_x,center_y,factor):

    gegl_graph_string="gegl:motion-blur-zoom center-x=%f center-y=%f factor=%f " % (center_x,center_y,factor)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_motion_blur_zoom",
    "GEGL to Python. Works on the active layer",
    "Zoom motion blur",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-motion-blur-zoom...",             #Menu path
    "*", 
    [
    (PF_FLOAT, "center_x","center-x (default:0.5, 0.0..1.0):", 0.5),
    (PF_FLOAT, "center_y","center-y (default:0.5, 0.0..1.0):", 0.5),
    (PF_FLOAT, "factor","factor (default:0.1, -0.5..1.0):", 0.1)
    ],
    [],
    gegl_motion_blur_zoom)

#---------------------------------------------------------------------------------------------
def gegl_noise_reduction(image, layer, iterations):

    gegl_graph_string="gegl:noise-reduction iterations=%d " % (iterations)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_noise_reduction",
    "GEGL to Python. Works on the active layer",
    "Anisotropic smoothing operation",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-noise-reduction...",             #Menu path
    "*", 
    [
    (PF_SPINNER, "iterations", "iterations (default:4, 0..8):", 4, (0, 8, 1))
    ],
    [],
    gegl_noise_reduction)

#---------------------------------------------------------------------------------------------
def gegl_open_buffer(image, layer, path):

    gegl_graph_string="gegl:open-buffer path=%s " % (path)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_open_buffer",
    "GEGL to Python. Works on the active layer",
    "Use an on-disk GeglBuffer as data source.",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-open-buffer...",             #Menu path
    "*", 
    [
    (PF_STRING, "path", "path(default:):", "")
    ],
    [],
    gegl_open_buffer)

#---------------------------------------------------------------------------------------------
def gegl_perlin_noise(image, layer, alpha,scale,zoff,n):

    gegl_graph_string="gegl:perlin-noise alpha=%f scale=%f zoff=%f n=%d " % (alpha,scale,zoff,n)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_perlin_noise",
    "GEGL to Python. Works on the active layer",
    "Perlin noise generator",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-perlin-noise...",             #Menu path
    "*", 
    [
    (PF_FLOAT, "alpha","alpha (default:1.2, 0.0..4.0):", 1.2),
    (PF_FLOAT, "scale","scale (default:1.8, 0.0..20.0):", 1.8),
    (PF_FLOAT, "zoff","zoff (default:-1.0, -1.0..8.0):", -1.0),
    (PF_SPINNER, "n", "n (default:3, 0..20):", 3, (0, 20, 1))
    ],
    [],
    gegl_perlin_noise)

#---------------------------------------------------------------------------------------------
def gegl_photocopy(image, layer, mask_radius,sharpness,black,white):

    gegl_graph_string="gegl:photocopy mask-radius=%f sharpness=%f black=%f white=%f " % (mask_radius,sharpness,black,white)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_photocopy",
    "GEGL to Python. Works on the active layer",
    "Simulate color distortion produced by a copy machine",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-photocopy...",             #Menu path
    "*", 
    [
    (PF_FLOAT, "mask_radius","mask-radius (default:10.0, 0.0..50.0):", 10.0),
    (PF_FLOAT, "sharpness","sharpness (default:0.5, 0.0..1.0):", 0.5),
    (PF_FLOAT, "black","black (default:0.2, 0.0..1.0):", 0.2),
    (PF_FLOAT, "white","white (default:0.2, 0.0..1.0):", 0.2)
    ],
    [],
    gegl_photocopy)

#---------------------------------------------------------------------------------------------
def gegl_polar_coordinates(image, layer, depth,angle,bw,top,polar,pole_x,pole_y,middle):

    gegl_graph_string="gegl:polar-coordinates depth=%f angle=%f bw=%d top=%d polar=%d pole-x=%d pole-y=%d middle=%d " % (depth,angle,bw,top,polar,pole_x,pole_y,middle)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_polar_coordinates",
    "GEGL to Python. Works on the active layer",
    "Convert image to or from polar coordinates",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-polar-coordinates...",             #Menu path
    "*", 
    [
    (PF_FLOAT, "depth","depth (default:100.0, 0.0..100.0):", 100.0),
    (PF_FLOAT, "angle","angle (default:0.0, 0.0..360.0):", 0.0),
    (PF_TOGGLE, "bw","bw (default:True):",True),
    (PF_TOGGLE, "top","top (default:True):",True),
    (PF_TOGGLE, "polar","polar (default:True):",True),
    (PF_SPINNER, "pole_x", "pole-x (default:0, 0..10000):", 0, (0, 10000, 1)),
    (PF_SPINNER, "pole_y", "pole-y (default:0, 0..10000):", 0, (0, 10000, 1)),
    (PF_TOGGLE, "middle","middle (default:True):",True)
    ],
    [],
    gegl_polar_coordinates)

#---------------------------------------------------------------------------------------------
def gegl_posterize(image, layer, levels):

    gegl_graph_string="gegl:posterize levels=%d " % (levels)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_posterize",
    "GEGL to Python. Works on the active layer",
    "Reduces the number of levels in each color component of the image.",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-posterize...",             #Menu path
    "*", 
    [
    (PF_SPINNER, "levels", "levels (default:8, 1..64):", 8, (1, 64, 1))
    ],
    [],
    gegl_posterize)

#---------------------------------------------------------------------------------------------
def gegl_red_eye_removal(image, layer, threshold):

    gegl_graph_string="gegl:red-eye-removal threshold=%f " % (threshold)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_red_eye_removal",
    "GEGL to Python. Works on the active layer",
    "Remove the red eye effect caused by camera flashes",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-red-eye-removal...",             #Menu path
    "*", 
    [
    (PF_FLOAT, "threshold","threshold (default:0.4, 0.0..0.8):", 0.4)
    ],
    [],
    gegl_red_eye_removal)

#---------------------------------------------------------------------------------------------
def gegl_reinhard05(image, layer, brightness,chromatic,light):

    gegl_graph_string="gegl:reinhard05 brightness=%f chromatic=%f light=%f " % (brightness,chromatic,light)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_reinhard05",
    "GEGL to Python. Works on the active layer",
    "Adapt an image, which may have a high dynamic range, for presentation using a low dynamic range. This is an efficient global operator derived from simple physiological observations, producing luminance within the range 0.0-1.0",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-reinhard05...",             #Menu path
    "*", 
    [
    (PF_FLOAT, "brightness","brightness (default:0.0, -100.0..100.0):", 0.0),
    (PF_FLOAT, "chromatic","chromatic (default:0.0, 0.0..1.0):", 0.0),
    (PF_FLOAT, "light","light (default:1.0, 0.0..1.0):", 1.0)
    ],
    [],
    gegl_reinhard05)

#---------------------------------------------------------------------------------------------
def gegl_rgb_clip(image, layer, clip_low,low_limit,clip_high,high_limit):

    gegl_graph_string="gegl:rgb-clip clip-low=%d low-limit=%f clip-high=%d high-limit=%f " % (clip_low,low_limit,clip_high,high_limit)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_rgb_clip",
    "GEGL to Python. Works on the active layer",
    "Keep RGB pixels values inside a specific range",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-rgb-clip...",             #Menu path
    "*", 
    [
    (PF_TOGGLE, "clip_low","clip-low (default:True):",True),
    (PF_FLOAT, "low_limit","low-limit (default:0.0, -2.0..0.0):", 0.0),
    (PF_TOGGLE, "clip_high","clip-high (default:True):",True),
    (PF_FLOAT, "high_limit","high-limit (default:1.0, 1.0..3.0):", 1.0)
    ],
    [],
    gegl_rgb_clip)

#---------------------------------------------------------------------------------------------
def gegl_sdl2_display(image, layer, window_title):

    gegl_graph_string="gegl:sdl2-display window-title=%s " % (window_title)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_sdl2_display",
    "GEGL to Python. Works on the active layer",
    "Displays the input buffer in an SDL2 window (restricted to one display op/process, due to SDL2 implementation issues).",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-sdl2-display...",             #Menu path
    "*", 
    [
    (PF_STRING, "window_title", "window-title(default:window_title):", "window_title")
    ],
    [],
    gegl_sdl2_display)

#---------------------------------------------------------------------------------------------
def gegl_sepia(image, layer, scale,srgb):

    gegl_graph_string="gegl:sepia scale=%f srgb=%d " % (scale,srgb)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_sepia",
    "GEGL to Python. Works on the active layer",
    "Apply a sepia tone to the input image",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-sepia...",             #Menu path
    "*", 
    [
    (PF_FLOAT, "scale","scale (default:1.0, 0.0..1.0):", 1.0),
    (PF_TOGGLE, "srgb","srgb (default:True):",True)
    ],
    [],
    gegl_sepia)

#---------------------------------------------------------------------------------------------
def gegl_shadows_highlights(image, layer, shadows,highlights,whitepoint,radius,compress,shadows_ccorrect,highlights_ccorrect):

    gegl_graph_string="gegl:shadows-highlights shadows=%f highlights=%f whitepoint=%f radius=%f compress=%f shadows-ccorrect=%f highlights-ccorrect=%f " % (shadows,highlights,whitepoint,radius,compress,shadows_ccorrect,highlights_ccorrect)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_shadows_highlights",
    "GEGL to Python. Works on the active layer",
    "Perform shadows and highlights correction",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-shadows-highlights...",             #Menu path
    "*", 
    [
    (PF_FLOAT, "shadows","shadows (default:0.0, -100.0..100.0):", 0.0),
    (PF_FLOAT, "highlights","highlights (default:0.0, -100.0..100.0):", 0.0),
    (PF_FLOAT, "whitepoint","whitepoint (default:0.0, -10.0..10.0):", 0.0),
    (PF_FLOAT, "radius","radius (default:100.0, 0.1..200.0):", 100.0),
    (PF_FLOAT, "compress","compress (default:50.0, 0.0..100.0):", 50.0),
    (PF_FLOAT, "shadows_ccorrect","shadows-ccorrect (default:100.0, 0.0..100.0):", 100.0),
    (PF_FLOAT, "highlights_ccorrect","highlights-ccorrect (default:50.0, 0.0..100.0):", 50.0)
    ],
    [],
    gegl_shadows_highlights)

#---------------------------------------------------------------------------------------------
def gegl_slic(image, layer, cluster_size,compactness,iterations):

    gegl_graph_string="gegl:slic cluster-size=%d compactness=%d iterations=%d " % (cluster_size,compactness,iterations)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_slic",
    "GEGL to Python. Works on the active layer",
    "Simple Linear Iterative Clustering. Superpixels based on k-means clustering",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-slic...",             #Menu path
    "*", 
    [
    (PF_SPINNER, "cluster_size", "cluster-size (default:32, 2..1024):", 32, (2, 1024, 1)),
    (PF_SPINNER, "compactness", "compactness (default:20, 1..40):", 20, (1, 40, 1)),
    (PF_SPINNER, "iterations", "iterations (default:1, 1..15):", 1, (1, 15, 1))
    ],
    [],
    gegl_slic)

#---------------------------------------------------------------------------------------------
def gegl_snn_mean(image, layer, radius,pairs):

    gegl_graph_string="gegl:snn-mean radius=%d pairs=%d " % (radius,pairs)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_snn_mean",
    "GEGL to Python. Works on the active layer",
    "Symmetric Nearest Neighbor.Noise reducing edge preserving blur filter based on Symmetric Nearest Neighbors ",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-snn-mean...",             #Menu path
    "*", 
    [
    (PF_SPINNER, "radius", "radius (default:8, 0..40):", 8, (0, 40, 1)),
    (PF_SPINNER, "pairs", "pairs (default:2, 1..2):", 2, (1, 2, 1))
    ],
    [],
    gegl_snn_mean)

#---------------------------------------------------------------------------------------------
def gegl_softglow(image, layer, glow_radius,brightness,sharpness):

    gegl_graph_string="gegl:softglow glow-radius=%f brightness=%f sharpness=%f " % (glow_radius,brightness,sharpness)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_softglow",
    "GEGL to Python. Works on the active layer",
    "Simulate glow by making highlights intense and fuzzy",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-softglow...",             #Menu path
    "*", 
    [
    (PF_FLOAT, "glow_radius","glow-radius (default:10.0, 1.0..50.0):", 10.0),
    (PF_FLOAT, "brightness","brightness (default:0.3, 0.0..1.0):", 0.3),
    (PF_FLOAT, "sharpness","sharpness (default:0.85, 0.0..1.0):", 0.85)
    ],
    [],
    gegl_softglow)

#---------------------------------------------------------------------------------------------
def gegl_stress(image, layer, radius,samples,iterations,enhance_shadows):

    gegl_graph_string="gegl:stress radius=%d samples=%d iterations=%d enhance-shadows=%d " % (radius,samples,iterations,enhance_shadows)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_stress",
    "GEGL to Python. Works on the active layer",
    "Spatio Temporal Retinex-like Envelope with Stochastic Sampling",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-stress...",             #Menu path
    "*", 
    [
    (PF_SPINNER, "radius", "radius (default:300, 2..1000):", 300, (2, 1000, 1)),
    (PF_SPINNER, "samples", "samples (default:5, 3..17):", 5, (3, 17, 1)),
    (PF_SPINNER, "iterations", "iterations (default:5, 1..30):", 5, (1, 30, 1)),
    (PF_TOGGLE, "enhance_shadows","enhance-shadows (default:True):",True)
    ],
    [],
    gegl_stress)

#---------------------------------------------------------------------------------------------
def gegl_stretch_contrast(image, layer, keep_colors,perceptual):

    gegl_graph_string="gegl:stretch-contrast keep-colors=%d perceptual=%d " % (keep_colors,perceptual)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_stretch_contrast",
    "GEGL to Python. Works on the active layer",
    "Scales the components of the buffer to be in the 0.0-1.0 range. This improves images that make poor use of the available contrast (little contrast, very dark, or very bright images).",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-stretch-contrast...",             #Menu path
    "*", 
    [
    (PF_TOGGLE, "keep_colors","keep-colors (default:True):",True),
    (PF_TOGGLE, "perceptual","perceptual (default:True):",True)
    ],
    [],
    gegl_stretch_contrast)

#---------------------------------------------------------------------------------------------
def gegl_stretch_contrast_hsv(image, layer, ):

    gegl_graph_string="gegl:stretch-contrast-hsv " % ()
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_stretch_contrast_hsv",
    "GEGL to Python. Works on the active layer",
    "Scales the components of the buffer to be in the 0.0-1.0 range. This improves images that make poor use of the available contrast (little contrast, very dark, or very bright images). This version differs from Contrast Autostretch in that it works in HSV space, and preserves hue.",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-stretch-contrast-hsv...",             #Menu path
    "*", 
    [

    ],
    [],
    gegl_stretch_contrast_hsv)

#---------------------------------------------------------------------------------------------
def gegl_svg_huerotate(image, layer, values):

    gegl_graph_string="gegl:svg-huerotate values=%s " % (values)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_svg_huerotate",
    "GEGL to Python. Works on the active layer",
    "SVG color matrix operation svg_huerotate",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-svg-huerotate...",             #Menu path
    "*", 
    [
    (PF_STRING, "values", "values(default:):", "")
    ],
    [],
    gegl_svg_huerotate)

#---------------------------------------------------------------------------------------------
def gegl_svg_luminancetoalpha(image, layer, ):

    gegl_graph_string="gegl:svg-luminancetoalpha " % ()
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_svg_luminancetoalpha",
    "GEGL to Python. Works on the active layer",
    "SVG color matrix operation svg_luminancetoalpha",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-svg-luminancetoalpha...",             #Menu path
    "*", 
    [

    ],
    [],
    gegl_svg_luminancetoalpha)

#---------------------------------------------------------------------------------------------
def gegl_svg_matrix(image, layer, values):

    gegl_graph_string="gegl:svg-matrix values=%s " % (values)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_svg_matrix",
    "GEGL to Python. Works on the active layer",
    "SVG color matrix operation svg_matrix",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-svg-matrix...",             #Menu path
    "*", 
    [
    (PF_STRING, "values", "values(default:):", "")
    ],
    [],
    gegl_svg_matrix)

#---------------------------------------------------------------------------------------------
def gegl_svg_saturate(image, layer, values):

    gegl_graph_string="gegl:svg-saturate values=%s " % (values)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_svg_saturate",
    "GEGL to Python. Works on the active layer",
    "SVG color matrix operation svg_saturate",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-svg-saturate...",             #Menu path
    "*", 
    [
    (PF_STRING, "values", "values(default:):", "")
    ],
    [],
    gegl_svg_saturate)


#---------------------------------------------------------------------------------------------
def gegl_tile_glass(image, layer, tile_width,tile_height):

    gegl_graph_string="gegl:tile-glass tile-width=%d tile-height=%d " % (tile_width,tile_height)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_tile_glass",
    "GEGL to Python. Works on the active layer",
    "Simulate distortion caused by rectangular glass tiles",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-tile-glass...",             #Menu path
    "*", 
    [
    (PF_SPINNER, "tile_width", "tile-width (default:25, 5..50):", 25, (5, 50, 1)),
    (PF_SPINNER, "tile_height", "tile-height (default:25, 5..50):", 25, (5, 50, 1))
    ],
    [],
    gegl_tile_glass)

#---------------------------------------------------------------------------------------------
def gegl_tile_seamless(image, layer, ):

    gegl_graph_string="gegl:tile-seamless " % ()
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_tile_seamless",
    "GEGL to Python. Works on the active layer",
    "Make the input buffer seamlessly tileable. The algorithm is not content-aware, so the result may need post-processing.",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-tile-seamless...",             #Menu path
    "*", 
    [

    ],
    [],
    gegl_tile_seamless)

#---------------------------------------------------------------------------------------------
def gegl_unpremultiply(image, layer, ):

    gegl_graph_string="gegl:unpremultiply " % ()
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_unpremultiply",
    "GEGL to Python. Works on the active layer",
    "Unpremultiplies a buffer that contains pre-multiplied colors (but according to the babl format is not.)",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-unpremultiply...",             #Menu path
    "*", 
    [

    ],
    [],
    gegl_unpremultiply)

#---------------------------------------------------------------------------------------------
def gegl_unsharp_mask(image, layer, std_dev,scale,threshold):

    gegl_graph_string="gegl:unsharp-mask std-dev=%f scale=%f threshold=%f " % (std_dev,scale,threshold)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_unsharp_mask",
    "GEGL to Python. Works on the active layer",
    "Sharpen image, by adding difference to blurred image, a technique for sharpening originally used in darkrooms.",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-unsharp-mask...",             #Menu path
    "*", 
    [
    (PF_FLOAT, "std_dev","std-dev (default:3.0, 0.0..40.0):", 3.0),
    (PF_FLOAT, "scale","scale (default:0.5, 0.0..10.0):", 0.5),
    (PF_FLOAT, "threshold","threshold (default:0.0, 0.0..1.0):", 0.0)
    ],
    [],
    gegl_unsharp_mask)

#---------------------------------------------------------------------------------------------
def gegl_value_invert(image, layer, ):

    gegl_graph_string="gegl:value-invert " % ()
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_value_invert",
    "GEGL to Python. Works on the active layer",
    "Invert the value component, the result has the brightness inverted, keeping the color.",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-value-invert...",             #Menu path
    "*", 
    [

    ],
    [],
    gegl_value_invert)

#---------------------------------------------------------------------------------------------
def gegl_wavelet_blur(image, layer, radius):

    gegl_graph_string="gegl:wavelet-blur radius=%f " % (radius)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_wavelet_blur",
    "GEGL to Python. Works on the active layer",
    "This blur is used for the wavelet decomposition filter, each pixel is computed from another by the HAT transform",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-wavelet-blur...",             #Menu path
    "*", 
    [
    (PF_FLOAT, "radius","radius (default:1.0, 0.0..256.0):", 1.0)
    ],
    [],
    gegl_wavelet_blur)

#---------------------------------------------------------------------------------------------
def gegl_whirl_pinch(image, layer, whirl,pinch,radius):

    gegl_graph_string="gegl:whirl-pinch whirl=%f pinch=%f radius=%f " % (whirl,pinch,radius)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_whirl_pinch",
    "GEGL to Python. Works on the active layer",
    "Distort an image by whirling and pinching",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-whirl-pinch...",             #Menu path
    "*", 
    [
    (PF_FLOAT, "whirl","whirl (default:90.0, -720.0..720.0):", 90.0),
    (PF_FLOAT, "pinch","pinch (default:0.0, -1.0..1.0):", 0.0),
    (PF_FLOAT, "radius","radius (default:1.0, 0.0..2.0):", 1.0)
    ],
    [],
    gegl_whirl_pinch)
	
#-------------------------------------MrQ-----------------------------------------------------	
def gegl_embos(image, layer, type,  azimuth,elevation,depth):

    if type==0:
        embos_type= "bumpmap"
    if type==1:
        embos_type= "emboss"
    gegl_graph_string="gegl:emboss type=%s azimuth=%d elevation=%d depth=%d " % (embos_type,azimuth, elevation, depth)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_embos",
    "GEGL to Python. Works on the active layer",
    "Simulates an image created by embossing",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-embos...",             #Menu path
    "*", 
    [

    (PF_OPTION,  "embos_type",  "Emboss type:", 0, ["Bumpmap","Emboss"]),
    (PF_SPINNER, "azimuth", 'Azimuth: ', 135, (0,360,1)),             # azimuth type: double default: 30.00 minimum: 0.00 maximum: 360.00 
    (PF_SPINNER, "elevation", 'Elevation: ', 45, (0,180,1)),  	     # elevation type: double default: 45.00 minimum: 0.00 maximum: 180.00 
    (PF_SPINNER, "depth", 'Depth: ', 6, (1,100,1)),  
    ],
    [],
    gegl_embos)	

#-------------------------------------MrQ-----------------------------------------------------	
def gegl_component_extract(image, layer, type,  invert,linear):
    

		
    if type==0:
        component_e= "rgb-r"
    if type==1:
        component_e= "rgb-g"
    if type==2:
        component_e= "rgb-b"
    if type== 3:
        component_e= "hue"
    if type==4 :
        component_e= "hsv-s"
    if type==5 :
        component_e= "hsv-v"
    if type== 6:
        component_e= "hsl-s"
    if type== 7:
        component_e= "hsl-l"
    if type== 8:
        component_e= "cmyk-c"
    if type== 9:
        component_e= "cmyk-m"
    if type== 10:
        component_e= "cmyk-y"
    if type==11 :
        component_e= "cmyk-k"
    if type==12 :
        component_e= "ycbcr-y",
    if type==13 :
        component_e= "ycbcr-cb"
    if type== 14:
        component_e= "ycbcr-cr"
    if type== 15:
        component_e= "lab-l"
    if type== 16:
        component_e= "lab-a"
    if type== 17:
        component_e= "lab-b"
    if type== 18:
        component_e= "lch-c"
    if type==19 :
        component_e= "lch-h"	
    if type==20:
        component_e= "alpha"		
		
		
    gegl_graph_string="gegl:component-extract component=%s invert=%d linear=%d " % (component_e, invert, linear)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_component_extract",
    "GEGL to Python. Works on the active layer",
    "Extract a color model component",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-component extract...",             #Menu path
    "*", 
    [

    (PF_OPTION,  "Component",  "Component type:", 20, [ "RGB Red",'RGB Green','RGB Blue','Hue','HSV Saturation','HSV Value','HSL Saturation','HSL Lightness','CMYK Cyan','CMYK Magenta','CMYK Yellow','CMYK Key','Y CbCr Y','Y CbCr Cb','Y CbCr Cr','LAB L','LAB A','LAB B','LCH Cab','LCH H ab','Alpha']),
    (PF_TOGGLE,  "invert",   "Invert? ",False), 
    (PF_TOGGLE,  "linear",   "Linear? ",False), 
    ],
    [],
    gegl_component_extract)	  

#-------------------------------------MrQ-----------------------------------------------------
def gegl_dropshadow(image, layer, shadow_offset_x, shadow_offset_y, shadow_blur, grow_shape, grow_radius, color_s, shadow_opacity):

    color_to_gegl=rgb_to_hex(color_s)
    if grow_shape==0:
        grow_s='square'
    if grow_shape==1:
        grow_s='circle'
    if grow_shape==2:
        grow_s='diamond'
		
    gegl_graph_string="gegl:dropshadow x=%d y=%d radius=%d grow-shape=%s grow-radius=%d color=%s opacity=%f" % (shadow_offset_x, shadow_offset_y, shadow_blur,grow_s,grow_radius, color_to_gegl, shadow_opacity)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_dropshadow",
    "GEGL to Python. Works on the active layer",
    "Creates a dropshadow effect on the input buffer",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-dropshadow...",             #Menu path
    "*", 
    [

    (PF_SPINNER, "shadow_offset_x", "Shadow Offset X:", 5, (-1000, 1000, 1)),
    (PF_SPINNER, "shadow_offset_y", "Shadow Offset Y:", 5, (-1000, 1000, 1)),
    (PF_SPINNER, "shadow_blur",     "Shadow Blur Radius:", 5, (0, 300, 1)),
    (PF_OPTION,  "grow_shape",      "Grow Shape: ", 0, [" Square "," Circle ", " Diamond "]),	
    (PF_SPINNER, "grow_radius",     "Grow Radius:", 5, (-100, 100, 1)),
    (PF_COLOR,   "color_s",         "Color:", (0, 0, 0)),
    (PF_FLOAT,   "shadow_opacity",  "Shadow Opacity (default:1.5, 0..2):", 1.5)      # (0, 2, 0.01)),  
    ],
    [],
    gegl_dropshadow)

#-------------------------------------MrQ-----------------------------------------------------
def gegl_supernova(image, layer,  color_sn, center_xs, center_ys ,radius_x, spokes_count):

	color_sn_to_gegl=rgb_to_hex(color_sn)
	gegl_graph_string="gegl:supernova center-x=%f center-y=%f radius=%d spokes-count=%d color=%s" % (center_xs, center_ys, radius_x, spokes_count, color_sn_to_gegl)
	pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)

register(
	"gegl_python_supernova",
    "GEGL to Python. Works on the active layer",
    "This plug-in produces an effect like a supernova burst. The amount of the light effect is approximately in proportion to 1/r, where r is the distance from the center of the star",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
	"<Image>/GEGL as Python/gegl-supernova...",             #Menu path
	"*", 
	[
	(PF_COLOR,   "color_sn",     "  Color:", (200,200,30)),		
	(PF_SPINNER, "center_xs",    "  Center X:", 0, (0, 1, 0.01)),		
	(PF_SPINNER, "center_ys",    "  Center Y:", 0, (0, 1, 0.01)),		
	(PF_SPINNER, "radius_x",     "  Radius:", 50, (1, 5000, 1)),
	(PF_SPINNER, "spokes_count", "  Number\n  of spokes:", 100, (1, 1024, 1)),
	
	],
	[],
	gegl_supernova)	
	
#-------------------------------------MrQ-----------------------------------------------------	
def gegl_mosaic(image, layer, tile_type, tile_size, tile_height, tile_neatness, color_variation, color_averaging, tile_surface, tile_allow_split, tile_spacing, joints_color, light_color, light_dir, antialiasing, seed ):


    if tile_type==0:
        tile_type_m= "squares"
    if tile_type==1:
        tile_type_m= "hexagons"
    if tile_type==2:
        tile_type_m= "octagons"
    if tile_type==3:
        tile_type_m= "triangles"
			
    joints_color_m=rgb_to_hex(joints_color)
    light_color_m=rgb_to_hex(light_color)
			
    if seed==0:
        seed_m=0
    if seed==1:
        seed_m=random.randrange(0,4294967296)	  

    #light_dir= light_dir_m/180.0 * math.pi
    gegl_graph_string="gegl:mosaic tile-type=%s tile-size=%f tile-height=%d tile-neatness=%f color-variation=%f color-averaging=%d tile-surface=%d tile-allow-split=%d tile-spacing=%f joints-color=%s light-color=%s light-dir=%s antialiasing=%s seed=%s" % (tile_type_m, tile_size,tile_height, tile_neatness, color_variation, color_averaging, tile_surface,tile_allow_split,tile_spacing, joints_color_m, light_color_m,light_dir,antialiasing, seed_m)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "gegl_python_mosaic",
    "GEGL to Python. Works on the active layer",
    "Mosaic is a filter which transforms an image into what appears to be a mosaic, composed of small primitives, each of constant color and of an approximate size.",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-mosaic...",             #Menu path
    "*", 
    [
    (PF_OPTION,  "tile_type",        "Tile type: ", 1, ["Squares", "Hexagons","Octagons", "Triangles"]),	
    (PF_SPINNER, "tile-size",        "Tile size:", 15, (1, 1000, 1)),	
    (PF_SPINNER, "tile_height",      "Tile height", 4, (1, 1000, 1)),	
    (PF_FLOAT, "tile_neatness",      "Tile neatness (default:0.65, 0..1)", 0.65),      #(0, 1, 0.01)),	
    (PF_FLOAT, "color_variation",    "Color variation (default:0.20, 0..1)", 0.20),    #(0, 1, 0.01)),

    (PF_TOGGLE,  "color_averaging",  "Color averaging",True),
    (PF_TOGGLE,  "tile_surface",     "Tile surface",   False),
    (PF_TOGGLE,  "tile-allow-split",  "Tile allow split",True),

	
    (PF_SPINNER, "tile_spacing",     "Tile spacing", 1, (0, 1000, 1)),	
	
    (PF_COLOR,   "joints_color",     "Joints color", (0,0,0)),		
    (PF_COLOR,   "light-color",      "Light color:", (256,256,256)),	
	
    (PF_SPINNER, "light-dir",        "Light dir", 135, (0, 360, 1)),	
    (PF_TOGGLE,  "antialiasing",     "Antialiasingn",True),
    (PF_OPTION,  "seed_m", "Seed:", 0, ["Seed=0","Random seed"]),		
	
    ],
    [],
    gegl_mosaic)

#-------------------------------------MrQ-----------------------------------------------------

# Bump Map: https://gegl.org/operations/gegl-bump-map.html 
# about GEGL graph by Kevin and Tmanni: https://www.gimp-forum.net/Thread-GEGL-graph?pid=19960#pid19960
# chanche offset: '+' up/right, '-' down/left
# gegl_graph_string + aux does not accept commas.

def gegl_bump_map(image, layer, save, type_b, compensate, invert, tiled, azimuth, elevation, depth, offset_x,  offset_y,  waterlevel, ambient):
   
    if os.name == 'nt':
        xchange = 'd:\image_1.png'
    else:
        xchange="/tmp/image_1.png"
    
    pdb.file_png_save(image, save, xchange, xchange, 1, 1, 1, 0, 0, 1, 0, run_mode=RUN_NONINTERACTIVE)
	
    if type_b==0:
        type= "linear"
    if type_b==1:
        type= "spherical"
    if type_b==2:
        type= "sinusoidal"

    if os.name == 'nt':
        gegl_graph_string="gegl:bump-map type=%s compensate=%g invert=%g tiled=%g azimuth=%d elevation=%f depth=%d offset-x=%d offset-y=%d waterlevel=%f ambient=%f aux=[ load path='d:\image_1.png' ]," % (type_b, compensate, invert, tiled, azimuth, elevation, depth, -offset_x,  -offset_y,  waterlevel, ambient)
    else:
        gegl_graph_string="gegl:bump-map type=%s compensate=%g invert=%g tiled=%g azimuth=%d elevation=%f depth=%d offset-x=%d offset-y=%d waterlevel=%f ambient=%f aux=[ load path='/tmp/image_1.png' ]," % (type_b, compensate, invert, tiled, azimuth, elevation, depth, -offset_x,  -offset_y,  waterlevel, ambient)
	
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
    os.remove(xchange)
	
register(
    "python_fu_gegl_bump_map",
    "GEGL to Python. Works on the active layer.\nAux bump-map must be the topmost layer.\nThe size of the layers must be identical.",
    "This plug-in uses the algorithm described by John Schlag, 'Fast Embossing Effects on Raster Image Data' in Graphics GEMS IV (ISBN 0-12-336155-9). It takes a buffer to be applied as a bump map to another buffer and produces a nice embossing effect.",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-bump-map...",             #Menu path
    "*", 
    [
		(PF_DRAWABLE,"save",        "Source Layer aux ➤\n(TOP LAYER)", None),		
		(PF_OPTION,  "type_b",      "Type:", 0, ["Linear","Spherical","Sinusoidal"]),	 	
		(PF_TOGGLE,  "compensate",  "Compensate?:",True),	
		(PF_TOGGLE,  "invert",      "Bump Invert? ",False),
		(PF_TOGGLE,  "tiled",       "Tiled ",False),	
		(PF_SPINNER, "azimuth",     "Azimuth: ", 45, (0, 360, 1)), 	
		(PF_FLOAT,   "elevation",   "Elevation (default:27, 0.5..90): ", 27), #(0.5, 90, 0.1)), 	
		(PF_SPINNER, "depth",       "Depth: ", 65, (1, 65, 1)),
		(PF_SPINNER, "offset_x",    "Offset x: ", 0, ( -20000, 20000, 1)),
		(PF_SPINNER, "offset_y",    "Offset Y: ", 0, ( -20000, 20000, 1)),	
		(PF_FLOAT,   "waterlevel",  "Waterlevel (default:0, 0.0..1.0): ", 0), #(0.0, 1, 0.1)), 	
		(PF_FLOAT,   "ambient",     "Ambient lighting\nfactor (default:0, 0...1): ", 0),		# (0, 1, 0.01)), 

    ],
    [],
    gegl_bump_map)
#-------------------------------------MrQ-----------------------------------------------------
def gegl_color_exchange(image, layer, from_color, to_color, red_threshold, green_threshold, blue_threshold):

    from_color_c=rgb_to_hex(from_color)
    to_color_c=rgb_to_hex(to_color)
		
    gegl_graph_string="gegl:color-exchange from-color=%s to-color=%s red-threshold=%f green-threshold=%f blue-threshold=%f " % ( from_color_c, to_color_c, red_threshold, green_threshold, blue_threshold )
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_color_exchange",
    "GEGL to Python. Works on the active layer",
    "Exchange one color with another, optionally setting a threshold to convert from one shade to another.",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-color-exchange...",             #Menu path
    "*", 
    [
    (PF_COLOR, "from_color",      "From Color:", (255, 255, 255)),	
    (PF_COLOR, "to_color",        "To Color:", (0, 0, 0)),	
    (PF_FLOAT, "red_threshold",   "Red Threshold  (default:0.0, 0.0..1.0):" , 0.0),
    (PF_FLOAT, "green_threshold", "Green Threshold (default:0.0, 0.0..1.0):", 0.0),
    (PF_FLOAT, "blue_threshold",  "Blue Threshold (default:0.0, 0.0..1.0):", 0.0),
 
    ],
    [],
    gegl_color_exchange)
#-------------------------------------MrQ-----------------------------------------------------	

def gegl_pixelize(image, layer, norm, size_x, size_y, offset_x, offset_y, ratio_x, ratio_y, background_p ):

    if norm==0:
        norm_p= "diamond"
    if norm==1:
        norm_p= "round"
    if norm==2:
        norm_p= "square"

    background=rgb_to_hex(background_p)
		
    gegl_graph_string="gegl:pixelize norm=%s size-x=%d size-y=%d offset-x=%d offset-y=%d ratio-x=%f ratio-y=%f background=%s " % ( norm_p, size_x, size_y, offset_x, offset_y, ratio_x, ratio_y, background )
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_pixelize",
    "GEGL to Python. Works on the active layer",
    "Simplify image into an array of solid-colored rectangles",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-pixelize...",             #Menu path
    "*", 
    [
    (PF_OPTION,  "norm_p",     "Shape", 0, ["Diamond","Round","Square"]),
    (PF_SPINNER, "size_x",     "Block width", 16, (1, 2048, 1)), 	
    (PF_SPINNER, "size_y",     "Block height", 16, (1, 2048, 1)), 	
    (PF_SPINNER, "offset_x",   "Offset X", 0, (0, 2048, 1)), 	
    (PF_SPINNER, "offset_x",   "Offset Y", 0, (0, 2048, 1)), 	
    (PF_FLOAT,   "ratio_x",    "Size ratio X (default:1.0, 0.0..1.0):", 0.5),
    (PF_FLOAT,   "ratio_y",    "Size ratio Y (default:1.0, 0.0..1.0):", 0.5),
    (PF_COLOR,   "background", "Background color:", (255,255,255)),
    ],
    [],
    gegl_pixelize)
	
#-------------------------------------MrQ-----------------------------------------------------

def gegl_newsprint_rgb(image, layer, pattern2, period2, angle2, pattern3, period3, angle3 ,pattern4, period4, angle4, aa_samples, turbulence, blocksize, angleboost):

    color_model="rgb"

    if pattern2==0:
        pattern2_n= "line",
    if pattern2==1:
        pattern2_n= "circle"
    if pattern2==2:
        pattern2_n= "diamond"
    if pattern2==3:
        pattern2_n= "pssquare"		
    if pattern2==4:
        pattern2_n= "cross"		

    if pattern3==0:
        pattern3_n= "line",
    if pattern3==1:
        pattern3_n= "circle"
    if pattern3==2:
        pattern3_n= "diamond"
    if pattern3==3:
        pattern3_n= "pssquare"		
    if pattern3==4:
        pattern3_n= "cross"
		
    if pattern4==0:
        pattern4_n= "line",
    if pattern4==1:
        pattern4_n= "circle"
    if pattern4==2:
        pattern4_n= "diamond"
    if pattern4==3:
        pattern4_n= "pssquare"		
    if pattern4==4:
        pattern4_n= "cross"		
	
    gegl_graph_string="gegl:newsprint color-model=%s pattern2=%s period2=%d angle2=%d pattern3=%s period3=%d angle3=%d pattern4=%s period4=%d angle4=%d aa-samples=%s turbulence=%f blocksize=%f angleboost=%f" % (color_model, pattern2_n, period2, angle2, pattern3_n, period3, angle3,pattern4_n, period4, angle4 ,aa_samples, turbulence, blocksize, angleboost )
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_newsprint_rgb",
    "GEGL to Python. Works on the active layer",
    "Digital halftoning with optional modulations.",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-newsprint-RGB...",             #Menu path
    "*", 
    [
    (PF_OPTION,  "pattern2",  "Red pattern", 0, ["Line","Circle","Diamond","Square (or Euclidian) dot","Crossing Lines"]),
    (PF_SPINNER, "period2",   "Red period", 12, (0, 200, 1)),
    (PF_SPINNER, "angle2",    "Red angle", 15, (-180, 180, 1)),
    (PF_OPTION,  "pattern3",  "Green pattern", 0, ["Line","Circle","Diamond","PSSquare (or Euclidian) dot","Crossing Lines"]),
    (PF_SPINNER, "period3",   "Green period", 12, (0, 200, 1)),
    (PF_SPINNER, "angle3",    "Green angle", 45, (-180, 180, 1)),
    (PF_OPTION,  "pattern4",  "Blue pattern", 0, ["Line","Circle","Diamond","PSSquare (or Euclidian) dot","Crossing Lines"]),
    (PF_SPINNER, "period4",   "Blue period", 12, (0, 200, 1)),
    (PF_SPINNER, "angle4",    "Blue angle", 0, (-180, 180, 1)),
    (PF_SPINNER, "aa-samples","Anti-alias\n oversampling factor", 16, (1, 128, 1)),	 	
    (PF_FLOAT,   "turbulence",  "Turbulence (default:0.0, 0.00..1.00):", 0.00),
    (PF_FLOAT,   "blocksize",   "Blocksize (default:-1.0, -1.00..64.00):", -1.00),
    (PF_FLOAT,   "angleboost",  "Angle Boost (default:0.0, 0.00..4.00):", 0.00),	
    ],
    [],
    gegl_newsprint_rgb)	
	
def gegl_newsprint_bw(image, layer, color_model, pattern, period, angle, aa_samples, turbulence, blocksize, angleboost):
 
    if color_model==0:
        color_model_n= "white-on-black"
    if color_model==1:
        color_model_n= "black-on-white"
		
    if pattern==0:
        pattern_n= "line",
    if pattern==1:
        pattern_n= "circle"
    if pattern==2:
        pattern_n= "diamond"
    if pattern==3:
        pattern_n= "pssquare"		
    if pattern==4:
        pattern_n= "cross"		
	
	
    gegl_graph_string="gegl:newsprint color-model=%s pattern=%s period=%d angle=%d aa-samples=%s turbulence=%f blocksize=%f angleboost=%f" % (color_model_n, pattern_n, period, angle, aa_samples, turbulence, blocksize, angleboost )
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_newsprint_bw",
    "GEGL to Python. Works on the active layer",
    "Digital halftoning with optional modulations.",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-newsprint-B&W...",             #Menu path
    "*", 
    [
    (PF_OPTION,  "color_model", "Color Model", 0, ["White on Black","Black on White"]),	
    (PF_OPTION,  "pattern",     "Pattern", 0, ["Line","Circle","Diamond","Square (or Euclidian) dot","Crossing Lines"]),
    (PF_SPINNER, "period",      "Period", 12, (0, 200, 1)),
    (PF_SPINNER, "angle",       "Angle", 75, (-180, 180, 1)),

    (PF_SPINNER, "aa-samples","Anti-alias\n oversampling factor", 16, (1, 128, 1)),	 	
    (PF_FLOAT,   "turbulence",  "Turbulence (default:0.0, 0.00..1.00):", 0.00),
    (PF_FLOAT,   "blocksize",   "Blocksize (default:-1.0, -1.00..64.00):", -1.00),
    (PF_FLOAT,   "angleboost",  "Angle Boost (default:0.0, 0.00..4.00):", 0.00),	
    ],
    [],
    gegl_newsprint_bw)	


def gegl_newsprint_cmyk(image, layer, pattern2, period2, angle2, pattern3, period3, angle3 ,pattern4, period4, angle4, pattern, period, angle, black_pullout, aa_samples, turbulence, blocksize, angleboost):

    color_model="cmyk"

    if pattern2==0:
        pattern2_n= "line",
    if pattern2==1:
        pattern2_n= "circle"
    if pattern2==2:
        pattern2_n= "diamond"
    if pattern2==3:
        pattern2_n= "pssquare"		
    if pattern2==4:
        pattern2_n= "cross"		

    if pattern3==0:
        pattern3_n= "line",
    if pattern3==1:
        pattern3_n= "circle"
    if pattern3==2:
        pattern3_n= "diamond"
    if pattern3==3:
        pattern3_n= "pssquare"		
    if pattern3==4:
        pattern3_n= "cross"
		
    if pattern4==0:
        pattern4_n= "line",
    if pattern4==1:
        pattern4_n= "circle"
    if pattern4==2:
        pattern4_n= "diamond"
    if pattern4==3:
        pattern4_n= "pssquare"		
    if pattern4==4:
        pattern4_n= "cross"		

    if pattern==0:
        pattern_n= "line",
    if pattern==1:
        pattern_n= "circle"
    if pattern==2:
        pattern_n= "diamond"
    if pattern==3:
        pattern_n= "pssquare"		
    if pattern==4:
        pattern_n= "cross"	
		
    gegl_graph_string="gegl:newsprint color-model=%s pattern2=%s period2=%d angle2=%d pattern3=%s period3=%d angle3=%d pattern4=%s period4=%d angle4=%d pattern=%s period=%d angle=%d black-pullout=%f aa-samples=%s turbulence=%f blocksize=%f angleboost=%f" % (color_model, pattern2_n, period2, angle2, pattern3_n, period3, angle3,pattern4_n, period4, angle4, pattern_n, period, angle, black_pullout, aa_samples, turbulence, blocksize, angleboost )
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_newsprint_cmyk",
    "GEGL to Python. Works on the active layer",
    "Digital halftoning with optional modulations.",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-newsprint-CMYK...",             #Menu path
    "*", 
    [
    (PF_OPTION,  "pattern2",    "Cyan pattern", 0, ["Line","Circle","Diamond","Square (or Euclidian) dot","Crossing Lines"]),
    (PF_SPINNER, "period2",     "Cyan period", 12, (0, 200, 1)),
    (PF_SPINNER, "angle2",      "Cyan angle", 15, (-180, 180, 1)),
    (PF_OPTION,  "pattern3",    "Magenta pattern", 0, ["Line","Circle","Diamond","PSSquare (or Euclidian) dot","Crossing Lines"]),
    (PF_SPINNER, "period3",     "Magenta period", 12, (0, 200, 1)),
    (PF_SPINNER, "angle3",      "Magenta angle", 45, (-180, 180, 1)),
    (PF_OPTION,  "pattern4",    "Yellow pattern", 0, ["Line","Circle","Diamond","PSSquare (or Euclidian) dot","Crossing Lines"]),
    (PF_SPINNER, "period4",     "Yellow period", 12, (0, 200, 1)),
    (PF_SPINNER, "angle4",      "Yellow angle", 0, (-180, 180, 1)),
    (PF_OPTION,  "pattern",     "Black Pattern", 0, ["Line","Circle","Diamond","Square (or Euclidian) dot","Crossing Lines"]),
    (PF_SPINNER, "period",      "Black Period", 12, (0, 200, 1)),
    (PF_SPINNER, "angle",       "Black Angle", 75, (-180, 180, 1)),
    (PF_FLOAT,   "black-pullout", "Black pullout (default:1.00, 0.00..1.00):", 1.00),
	
    (PF_SPINNER, "aa-samples",  "Anti-alias\n oversampling factor", 16, (1, 128, 1)),	 	
    (PF_FLOAT,   "turbulence",  "Turbulence (default:0.0, 0.00..1.00):", 0.00),
    (PF_FLOAT,   "blocksize",   "Blocksize (default:-1.0, -1.00..64.00):", -1.00),
    (PF_FLOAT,   "angleboost",  "Angle Boost (default:0.0, 0.00..4.00):", 0.00),	
    ],
    [],
    gegl_newsprint_cmyk)

#-------------------------------------MrQ-----------------------------------------------------

def gegl_long_shadow(image, layer, style, angle, length, color_ls, composition):

    color=rgb_to_hex(color_ls)
    if style==0:
        style_ls= "finite"
    if style==1:
        style_ls= "infinite"
    if style==2:
        style_ls= "fading"
    if style==3:
        style_ls= "fading-fixed-length"	

    if composition==0:
        composition_ls= "shadow-plus-image"
    if composition==1:
        composition_ls= "shadow-only"
    if composition==2:
        composition_ls= "shadow-minus-image"
					
    gegl_graph_string="gegl:long-shadow style=%s angle=%d length=%d color=%s composition=%s " % (style_ls, angle, length, color, composition_ls)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)
register(
    "python_fu_gegl_long_shadow",
    "GEGL to Python. Works on the active layer",
    "Creates a long-shadow effect",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-long-shadow...",             #Menu path
    "*", 
    [
    (PF_OPTION,  "style",       "Style:", 0, ["Finite","Infinite","Fading","Fading (fixed length)"]),		
    (PF_SPINNER, "angle",       "Angle: ", 45, (-180, 180, 1)),
    (PF_SPINNER, "length",      "Length:", 100, (0, 1000, 1)),		
    (PF_COLOR,   "color_ls",    "Color:", (0,0,0)),	
    (PF_OPTION,  "composition", "Composition:", 0, ["Shadow plus image","Shadow only","Shadow minus image"]), 
    ],
    [],
    gegl_long_shadow)
#-------------------------------------MrQ-----------------------------------------------------
def gegl_cubism(image, layer,tile_size, tile_saturation, bg_color ,seed):
    if seed==0:
        seed_m=0
    if seed==1:
        seed_m=random.randrange(0,4294967296)
    bg_color_m=rgb_to_hex(bg_color)
		
    gegl_graph_string="gegl:cubism tile-size=%f tile-saturation=%f bg-color=%s seed=%f" % (tile_size, tile_saturation, bg_color_m ,seed_m)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)

register(
    "python_fu_gegl_cubism",
    "GEGL to Python. Works on the active layer.\nReguires glossy_bevel-script-only.dll/so",
    "Convert the image into randomly rotated square blobs, somehow resembling a cubist painting style.",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/gegl-cubism...",             #Menu path
    "*",
    [
    (PF_FLOAT, "tile-size","Tile size (default:10.0, 0.0..256.0):", 10.0),
    (PF_FLOAT, "tile-saturation","Tile saturation (default:2.5, 0.0..10.0):", 2.5),
    (PF_COLOR, "bg_color","Background color", (0,0,0)),
    (PF_OPTION,  "seed_m", "Seed:", 0, ["Seed=0","Random seed"]),	
    ],
    [],
    gegl_cubism)	
main()

